	title autotrack driver - KCT El and Az to a parallel port, manual part
;
;	Courtney Duncan, N5BF, 14 Dec 88, KCT version 9 January 90
;
;	This program runs from the command line and makes the appropriate
;	calls to the interrupt normally used for KCT control.  Also, some 
;	configuration utilities are available from this call.
;
;	Command line format:
;
;	stow m az el
;	stow e el el
;	stow a az az
;	stow i
;	stow c {+/-}az {+/-}el
;
;		'm' means move.
;		az and el are whole integers in the range 000-360 and
;		000-180 respectively.  The 'i' option causes the antennas
;		to be set to track on 'their backs.'  The 'c' option
;		sets small mechanical corrections (recommend just a few
;		degrees of each, if not zero, which is ideal).  Plus or
;		minus signs are required on this option.  The 'a' option
;		sets the (pre corrected) stall limits on azimuth.  The
;		'e' option sets the (pre corrected) stall limits on 
;		elevation.  This is an open loop way of preventing travel
;		limit stalls in KR-5400 systems.
;
;		During setup, use stow c +0 +0, then find the limits by
;		giving various commands until the system stalls.  Place
;		the determined limits in autoexec files.  Defaults are
;		180 (low) to 179 (high) for azimuth and 0 to 180 for
;		elevation.  If commands attempt to exceed the limits,
;		only the limit values will be commanded.
;
;	This program checks for the TSR signature, if it is not,
;	a command line error is issued.
;
;	When calling to interrupt 63h the following usage occurs.
;
;	AH=3	set the rotators to some position
;	CX	elevation, signed binary 
;	DX	azimuth, signed binary
;

int_at	equ	63h	;interrupt service routine, aka 68h

rtrmov	equ	04d	;by KCT standard
invflp	equ	96d	;user number for inverted mode toggle
mechset	equ	97d	;user number for mechanical errors set
stlste	equ	98d	;user number for elevation stall limit set
stlsta	equ	99d	;user number for azimuth stall limit set

;
;	macros
;

conout	macro message			;console output
	mov	dx,offset message	;message string, '$' terminator
	mov	ah,09h			;CP/M like system call
	int	21h			;invoke DOS
	endm


code	segment	public

	org	100h			;who knows why they put us 
	assume	cs:code,ds:code,es:code	; through this

start:
	push	cs
	pop	ds			;so the assume statement works

	mov	at_i,int_at		;software interrupt vector

;
;	get stuff from command line
;
;	case - commanding inversion:
;	ds:81h - space
;	ds:82h - 'i'
;	ds:83h - and beyond are ignored
;
;	case - setting mechanical corrections:
;	ds:81h - space
;	ds:82h - 'c'
;	ds:83h - space
;	ds:84h - '-' or '+'
;	ds:85h - number 0 - 9 for azimuth correction
;	ds:86h - space
;	ds:87h - '-' or '+'
;	ds:88h - number 0 - 9 for elevation correction
;	ds:89h - and beyond are ignored
;
;	case - commanding antenna movement
;	ds:81h - space
;	ds:82h - 'm' 'a' or 'e'
;	ds:83h - space
;	ds:84h - number 0 - 3, azimuth hundreds
;	ds:85h - number 0 - 9, azimuth tens
;	ds:86h - number 0 - 9, azimuth ones
;	ds:87h - space
;	ds:88h - number 0 - 9, elevation hundreds
;	ds:89h - number 0 - 9, elevation tens
;	ds:8Ah - number 0 - 9, elevation ones
;	ds:8Bh - and beyond are ignored
;

	xor	ah,ah			;want to be able to interchange ah, ax
	mov	bx,81h			;1st byte should be a space
	mov	al,[bx]
	cmp	al,' '
	je	prse
	jmp	cline_bad

prse:	inc	bx
	mov	al,[bx]			;2nd byte
prse0:	or	al,20h			;make case insensitive (change to lower)
	mov	lncmd,al
	cmp	al,'i'			;or must be the 'on-back' command
	jne	prse0i
	jmp	go
prse0i:	cmp	al,'c'			;or must be the 'corrections' command
	jne	prse0c
	jmp	prse8
prse0c:	cmp	al,'m'			;or move command
	jne	prse0m
	jmp	prse3
prse0m:	cmp	al,'e'			;or elevation limit command
	jne	prse0e
	jmp	prse3
prse0e:	cmp	al,'a'			;or azimuth limit command
	jne	prse0a
	jmp	prse3
prse0a:	jmp	cline_bad

prse8:	inc	bx			;3rd byte, finish the corrections case
	mov	al,[bx]
	cmp	al,' '
	je	prse9
	jmp	cline_bad		;must be a space

prse9:	inc	bx			;4th byte, sign of azimuth correction
	mov	al,[bx]
	mov	ngt,0			;means "positive"
	cmp	al,'+'
	je	prse2
	cmp	al,'-'
	je	prse10
	jmp	cline_bad		;must be '+' or '-'
prse10:	mov	ngt,1			;means "negative"

prse2:	inc	bx			;5th byte, value of azimuth correction
	mov	al,[bx]
	sub	al,'0'
	jnl	prse11
	jmp	cline_bad
prse11:	cmp	al,9d
	jng	prse12
	jmp	cline_bad
prse12:	cmp	ngt,0			;act according to given sign
	je	prse4
	neg	ax
prse4:	mov	azcmd,ax		;store the azimuth correction value
	
	inc	bx			;6rd byte
	xor	ah,ah			;reclear for ax, al interchangability
	mov	al,[bx]
	cmp	al,' '
	je	prse13
	jmp	cline_bad		;must be a space

prse13:	inc	bx			;7th byte, sign of elevation correction
	mov	al,[bx]
	mov	ngt,0			;means "positive"
	cmp	al,'+'
	je	prse5
	cmp	al,'-'
	je	prse14
	jmp	cline_bad		;must be '+' or '-'
prse14:	mov	ngt,1			;means "negative"

prse5:	inc	bx			;8th byte, value of elevation correction
	mov	al,[bx]
	sub	al,'0'
	jnl	prse15
	jmp	cline_bad
prse15:	cmp	al,9d
	jng	prse16
	jmp	cline_bad
prse16:	xor	ah,ah			;store the value if within range
	cmp	ngt,0			;act according to given sign
	je	prse6
	neg	ax
prse6:	mov	elcmd,ax		;store the elevation correction value
	mov	ch,lncmd		;user number for mechanical corrections
	jmp	go

prse3:	inc	bx			;3rd byte, finish stall or move case
	mov	al,[bx]
	cmp	al,' '
	jne	prse3s
	jmp	prse3a
prse3s:	jmp	cline_bad		;must be a space
prse3a:	inc	bx			;4th byte, hundres of azimuth
	mov	al,[bx]
	sub	al,'0'			;binary convert, valid check, must be 
	jnl	prse30
	jmp	cline_bad		; between 1 and 3 inclusive
prse30:	cmp	al,'3'			; for antenna move command
	jng	prse33
	jmp	cline_bad
prse33:	mov	azcmd,ax		;stored, hundreds of azimuth degrees
	inc	bx			;5th byte, tens of azimuth
	mov	al,[bx]
	sub	al,'0'			;convert to binary
	jnl	prse40
	jmp	cline_bad
prse40:	cmp	al,9d
	jng	prse49
	jmp	cline_bad
prse49:	xor	ah,ah
	push	ax
	mov	ax,azcmd
	mov	dx,10d
	mul	dx
	pop	dx
	add	ax,dx
	mov	azcmd,ax		;tens of azimuth
	
	inc	bx			;6th byte, ones of degrees, azimuth
	mov	al,[bx]
	sub	al,'0'
	jl	cline_bad
	cmp	al,9d
	jg	cline_bad
	xor	ah,ah
	push	ax
	mov	ax,azcmd
	mov	dx,10d
	mul	dx			;this could have a nonzero ah
	pop	dx
	add	ax,dx
	cmp	ax,360d
	jg	cline_bad		;pure range check, zero already checked
	mov	azcmd,ax		;azimuth finished

	inc	bx			;7th byte, space
	mov	al,[bx]
	cmp	al,' '
	jne	cline_bad

	xor	ah,ah
	inc	bx			;8th byte, hundreds degrees of elevation
	mov	al,[bx]
	sub	al,'0'
	jl	cline_bad
	cmp	al,1d
	jg	cline_bad
	mov	elcmd,ax		;partial result

	inc	bx			;9th byte, tens of degrees of elevation
	mov	al,[bx]
	sub	al,'0'
	jl	cline_bad
	cmp	al,9d
	jg	cline_bad
	push	ax
	mov	ax,elcmd
	mov	dx,10d
	mul	dx
	pop	dx
	add	ax,dx
	mov	elcmd,ax		;partial result

	inc	bx			;Ath byte, ones of degrees of elevation
	mov	al,[bx]
	sub	al,'0'
	jl	cline_bad
	cmp	al,9d
	jg	cline_bad
	push	ax
	mov	ax,elcmd
	mov	dx,10d
	mul	dx
	pop	dx
	add	ax,dx
	cmp	ax,180d			;check absolute range
	jg	cline_bad
	mov	elcmd,ax		;final elevation result
	xor	ch,ch			;user number 0 for direct command
	jmp	go

cline_bad:
	conout	usage
quit:	conout	noload
	int	20h			;return to dos, old style
no_int:	conout	noins
	int	20h			;return to dos, old style

;
;	the general data declaration area
;

at_i	db	?			;value of interrupt number
azcmd	dw	?			;number given for azimuth
elcmd	dw	?			;number given for elevation
ngt	db	?			;holds negative indication in parsing
lncmd	db	?			;holds line command adjust

;
;	stuff to print
;

noload	db	0Dh,0Ah,'stow aborted, bad arguments',0Dh,0Ah,'$'
usage	db	0Dh,0Ah,'command line must be either:'
	db	0Dh,0Ah,' '
	db	0Dh,0Ah,'   stow m AAA EEE'
	db	0Dh,0Ah,'   stow e AAA EEE'
	db	0Dh,0Ah,'   stow a AAA EEE'
	db	0Dh,0Ah,'       or'
	db	0Dh,0Ah,'   stow c {+/-}A {+/-}E'
	db	0Dh,0Ah,'       or'
	db	0Dh,0Ah,'   stow i'
	db	0Dh,0Ah,'       where'
	db	0Dh,0Ah,'        AAA is three digit azimuth (000, 360, etc.)'
	db	0Dh,0Ah,'        EEE is three digit elevation (000, 180, etc.)'
	db	0Dh,0Ah,'        A is single digit az mechanical correction'
	db	0Dh,0Ah,'        E is single digit el mechanical correction'
	db	0Dh,0Ah,'        c means "mechanical" corrections'
	db	0Dh,0Ah,'        m means "move"'
	db	0Dh,0Ah,'        a means "azimuth" stall limits'
	db	0Dh,0Ah,'        e means "elevation" stall limits'
	db	0Dh,0Ah,'        i means "invert" (track over backwards)'
	db	0Dh,0Ah,'$'
noins	db	0Dh,0Ah,'control interrupt not installed (run at.com)'
	db	0Dh,0Ah,'$'
sig	db	'RotorDRV'

go:	push	cx			;save the user number

;
;	check the interrupt installed indicator for the 'signature'
;
	push	es			;answer returned in es:bx, want to save
	mov	ah,35h
	mov	al,at_i
	int	21h
	add	bx,10d			;offset to the signature
	mov	di,bx			;now have es:di on signature
	mov	si,offset sig		;and ds:si on compare string here
	mov	cx,8			;string 8 bytes long
	cld				;going up...
	repne	cmpsb			;compare strings
	pop	es
	je	go1
	pop	cx			;restore stack before leaving
	jmp	no_int			;failure

go1:	pop	cx
	cmp	lncmd,'m'
	jne	go1m
	mov	ah,rtrmov		;the KCT set target command
	jmp	short	go0
go1m:	cmp	lncmd,'c'
	jne	go1c
	mov	ah,mechset
	jmp	short	go0
go1c:	cmp	lncmd,'e'
	jne	go1e
	mov	ah,stlste
	jmp	short	go0
go1e:	cmp	lncmd,'a'
	jne	go1a
	mov	ah,stlsta
	jmp	short	go0
go1a:	mov	ah,invflp		;all that is left is invert case
	int	int_at			;that's all, perform
	int	20h			;and exit

go0:	mov	cx,elcmd		;for any other case, load numbers
	mov	dx,azcmd
	int	int_at			;perform
	int	20h			;and exit

code	ends

	end	start			;the routine to run when run

